package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import java.util.Arrays;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findAllEmployeeEntities();
            findEmployeeNames();
            findEmployeeNamesAndIds();
            findEmployeeNamesAndSalaries();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 2000, "IT");
        Employee employee2 = Employee.create("Rose", 4500, "Admin");
        Employee employee3 = Employee.create("Denise", 2000, "Admin");
        Employee employee4 = Employee.create("Mike", 4000, "IT");
        Employee employee5 = Employee.create("Linda", 4500, "Sales");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.persist(employee5);
        em.getTransaction().commit();
        em.close();
    }

    private static void findAllEmployeeEntities() {
        System.out.println("-- finding Employee entities - entity selection --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Employee> query = cb.createQuery(Employee.class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.select(employee);//entity selection
        TypedQuery<Employee> typedQuery = em.createQuery(query);
        List<Employee> resultList = typedQuery.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeNames() {
        System.out.println("-- Employee names - single field selection --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<String> query = cb.createQuery(String.class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.select(employee.get(Employee_.name));//query selection ('select x, y, ... from' part)
        TypedQuery<String> typedQuery = em.createQuery(query);
        List<String> resultList = typedQuery.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeNamesAndIds() {
        System.out.println("-- Employee names and ids - multi selection using select() + array() --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Object[]> query = cb.createQuery(Object[].class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.select(cb.array(employee.get(Employee_.id), employee.get(Employee_.name)));//multiple selection
        TypedQuery<Object[]> typedQuery = em.createQuery(query);
        List<Object[]> resultList = typedQuery.getResultList();
                  resultList.forEach(arr -> System.out.println(Arrays.toString(arr)));
        em.close();
    }

    private static void findEmployeeNamesAndSalaries() {
        System.out.println("-- Employee names and salaries - multi selection using multiselect() --");
        EntityManager em = entityManagerFactory.createEntityManager();
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Object[]> query = cb.createQuery(Object[].class);//query object
        Root<Employee> employee = query.from(Employee.class);//from part
        query.multiselect(employee.get(Employee_.name), employee.get(Employee_.salary));//multiple selection
        TypedQuery<Object[]> typedQuery = em.createQuery(query);
        typedQuery.getResultList()
                  .forEach(arr -> System.out.println(Arrays.toString(arr)));
        em.close();
    }
}